
#include "gba_types.h"
#include "gba_mem.h"
#include "gba_const.h"
#include "aux.h"
#include <stdlib.h>
#include <stdio.h>

#define ROM_PROGRAM_LEN 16*1024
#ifdef MULTIBOOT
  volatile const u8 __gba_multiboot; // crt0.o const for multiboot mode
  #define ROM_ADRESS 0x2000000
#else
  #define ROM_ADRESS 0x8000000
#endif
#define ROM_DATA_AREA (ROM_ADRESS + ROM_PROGRAM_LEN)

#include "data_read.h"

extern void VBlankIntrWait();

#define REG_VCOUNT      *(volatile u16*)0x4000006

void vsync() {
    while(REG_VCOUNT >= 160);   // wait till VDraw
    while(REG_VCOUNT < 160);    // wait till VBlank
}

OAMEntry sprites[128];
pRotData rotData = (pRotData)sprites;

typedef struct Character 
{
	s32 x, y;
	u32 frame;
	u32 xblock, yblock;
	s32 xspeed, yspeed;
	s32 xaccel, yaccel;
	s32 xpos, ypos;
	u32 width, height;
	u8 land;
	u8 bounce;
}Character;

#include "bg_scroll.h"
#include "init.h"
#include "trans.h"
Character Player;

#include "character.h"


int AgbMain() {
	int i;
	u16 * pal = (u16*)BACKGROUND_PALETTE;

	for (i = 0; i < 100; i++) ColorScreen(0x0000);

	PrintLogo();

	DISPLAY_CONTROL = OBJECTS | MODE0 | BACKGROUND2 | SPRITE_1D;
	REG_VCOUNT |= (1 << 3);   //enable Vblank flag
	BACKGROUND2_CONTROL = BACKGROUND_WHICH_CHARACTER(3) | BACKGROUND_WHICH_MAP(0);
	BACKGROUND2_X_SCROLL = 0;
	BACKGROUND2_Y_SCROLL = 0;

	ViewportCenter(120,80);

	u8 * background = (u8*)GetFilePointer("level1.level");
	memcpy( (u8 *)0x5000000, &background[0], 32);
	u16 num_tiles = *(u16*)&background[32];
	u16 twidth = *(u16*)&background[34];
	u16 theight = *(u16*)&background[36];
	memcpy ((u16 *)BACKGROUND_CHARACTER(3),&background[64], num_tiles<<5);
	for (i = 0 ; i < 32; i++) memcpy ((u16 *)(BACKGROUND_MAP(0) + 32*i) , &background[64+(num_tiles<<5)+(2*i*twidth)], 64); //copy only the top-left
	
	u8 * palette_sonic = (u8*)GetFilePointer("sonic_run.sprite");
	u16 * bitmap_sonic = (u16*)&palette_sonic[32];
	memcpy( (u8 *)0x5000200, palette_sonic, 32);
	memcpy ( (u8*)(0x6010000), &bitmap_sonic[0], 256*32/2);

	trans_identity(0);
//	pal[0]=0xffff;  //the background-of-all color (non drawn pixels)
	pal[0]=color16(0,64,128);

	u16 * datap = (u16*)&background[64+(num_tiles<<5)];
	u16 * charblock = (u16 *)(BACKGROUND_MAP(0));

	Player.yaccel = Player.xaccel = 0;
	Player.yspeed = Player.xspeed = 0;
	Player.ypos = 70 << 16;
	Player.xpos = 30 << 16;
	Player.x = 30;
	Player.y = 70;
	Player.width = Player.height = 32;

	i = 0;
	while (1) {
		PlayerPhysics();

		vsync();

		BGScroll(datap,charblock,64,64);
	
		sprites[0].attribute0 = (COLOR_16 | SQUARE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width/2)));
		sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width/2)) );
		sprites[0].attribute2 = (i / 5 )<<4;
		i++;
		if ((i / 5 ) >= 8) i = 0;

	
		u32 loop;
		u16* temp;
		temp = (u16*)sprites;
		for(loop = 0;loop < 128*4; loop++) {
			OAMmem[loop] = temp[loop];
		}
	}

	return 0;
}

