

#include "character.h"
#include "gba_types.h"
#include "gba_mem.h"
#include "gba_const.h"

#include "level_loader.h"
#include "trans.h"

extern Character Player;
extern VIEWPORT vp;
Character LastPlayer;

/*  This functions control the input and the friction of the air and other speed modifications  */
void SAir() {
	if (~KEYPAD & KEY_LEFT) Player.xspeed-= SPEED_FACTOR_AIR;
	if (~KEYPAD & KEY_RIGHT) Player.xspeed+= SPEED_FACTOR_AIR;
	if (abs(Player.xspeed) < (SPEED_FACTOR)*20) Player.xspeed -= Player.xspeed >> 5;
	Player.yspeed -= Player.yspeed >> 5;
}

// If we got highspeed mantain it only if we are pressing the button
// if not pressing the button brake a little.
// If we havent got highspeed just set up a limit speed
void SLand() {
	if (abs(Player.xspeed) > (SPEED_FACTOR)*30 && (Player.xspeed) > 0) {
		if (~KEYPAD & KEY_LEFT) Player.xspeed-= SPEED_FACTOR*4;
		if (!(~KEYPAD & KEY_RIGHT)) Player.xspeed -= Player.xspeed >> 6;
	}else if (abs(Player.xspeed) > (SPEED_FACTOR)*30 && (Player.xspeed) < 0) {
		if (~KEYPAD & KEY_RIGHT) Player.xspeed+= SPEED_FACTOR*4;
		if (!(~KEYPAD & KEY_LEFT)) Player.xspeed -= Player.xspeed >> 6;
	}else{
		if (~KEYPAD & KEY_LEFT) Player.xspeed-= SPEED_FACTOR;
		if (~KEYPAD & KEY_RIGHT) Player.xspeed+= SPEED_FACTOR;
		Player.xspeed -= Player.xspeed >> 5;
	}

}
void SDiag1() {
	if (~KEYPAD & KEY_LEFT) Player.xspeed-= SPEED_FACTOR/8;
	if (~KEYPAD & KEY_RIGHT) Player.xspeed+= SPEED_FACTOR;
	if (Player.xspeed > 0) Player.xspeed -= Player.xspeed >> 7;
	else Player.xspeed -= Player.xspeed >> 6;
}
void SDiag2() {
	if (~KEYPAD & KEY_LEFT) Player.xspeed-= SPEED_FACTOR;
	if (~KEYPAD & KEY_RIGHT) Player.xspeed+= SPEED_FACTOR/8;
	if (Player.xspeed < 0) Player.xspeed -= Player.xspeed >> 7;
	else Player.xspeed -= Player.xspeed >> 6;
}

void SCurve1 () {SDiag1();}
void SCurve2 () {SDiag2();}

void (*SpeedControl[12])()  = {
	//0       1       2         3
	&SAir, &SLand, &SDiag1, &SDiag2,
	//4      5        6         7         //Inclinations1-4
	&SLand, &SLand, &SLand,  &SLand,
	//8      9        10        11        //Curves 1-4
	&SCurve1, &SCurve2, &SCurve1, &SCurve2
};

void PlayerPhysics(tlevel * level) {
	//there is a max accel. given by the user. 
	//other accel may exist due tu gravity or highspeed ways
	LastPlayer.xpos = Player.xpos;
	LastPlayer.ypos = Player.ypos;

	SpeedControl[Player.on_land]();
	
	if (!( (~KEYPAD & KEY_LEFT) || (~KEYPAD & KEY_RIGHT) || (~KEYPAD & KEY_DOWN) || (~KEYPAD & KEY_UP) ) ) {
		if (abs(Player.yspeed) < NEARLY_QUIET) Player.yspeed = 0;
		if (abs(Player.xspeed) < NEARLY_QUIET) Player.xspeed = 0;
	}
	if (Player.xspeed > 0) Player.orientation = 1;
	if (Player.xspeed < 0) Player.orientation = -1;

	Player.xspeed += Player.xaccel;
	Player.yspeed += Player.yaccel;
	Player.yspeed2 += GRAVITY;
	if (Player.yspeed2 > MAX_GRAVITY_SPEED) Player.yspeed2 = MAX_GRAVITY_SPEED;

	PlayerCollision(level);
	
	//Prevent a displace greater than 8 pixels (avoids tile copy faults)
	if (Player.xpos + Player.xspeed - LastPlayer.xpos > (8<<16)) Player.xspeed = LastPlayer.xpos - Player.xpos + (8<<16);
	if (Player.xpos + Player.xspeed - LastPlayer.xpos < -(8<<16)) Player.xspeed = LastPlayer.xpos - Player.xpos - (8<<16);

	if (Player.ypos + Player.yspeed + Player.yspeed2 - LastPlayer.ypos > (8<<16)) Player.yspeed = LastPlayer.ypos - Player.ypos - Player.yspeed2 + (8<<16);
	if (Player.ypos + Player.yspeed + Player.yspeed2 - LastPlayer.ypos < -(8<<16)) Player.yspeed = LastPlayer.ypos - Player.ypos - Player.yspeed2 - (8<<16);


	Player.xpos += Player.xspeed;
	Player.ypos += Player.yspeed + Player.yspeed2;
	
	Player.x = Player.xpos>>16;
	Player.y = Player.ypos>>16;
	
	//check the viewport and update it:
	if (vp.x - Player.x > 40) vp.x = Player.x + 40;
	if (Player.x - vp.x > 40) vp.x = Player.x - 40;
	
	if (vp.y - Player.y > 30) vp.y = Player.y + 30;
	if (Player.y - vp.y > 30) vp.y = Player.y - 30;
}

void PlayerCollision (tlevel * level) {
	//The player is in one, two or four block
	u32 tx = (Player.x/32);
	u32 ty = (Player.y/32);
	u32 tx2 = (Player.x + 16) / 32;
	if (tx2 == tx) tx2--;
	u32 ty2 = (Player.y + 16) / 32;
	if (ty2 == ty) ty2--;

	ProcessBlock[level->collision[tx+ty*level->width32]](tx,ty);
	ProcessBlock[level->collision[tx2+ty*level->width32]](tx2,ty);
	ProcessBlock[level->collision[tx2+ty2*level->width32]](tx2,ty2);
	ProcessBlock[level->collision[tx+ty2*level->width32]](tx,ty2);
}

#define WALL_MARGIN 10
#define ROOF_MARGIN 4

void (*ProcessBlock[20])(u32 tx, u32 ty)  = {
	&BlankBlock, &LeftWall, &Roof, &RightWall,
	&Land, &MidLand,
	&Corner1, &Corner2, &Corner3, &Corner4,
	&Diag1,	&Diag2,
	&Curve1, &Curve2, &Curve3, &Curve4,
	&Inclination1, &Inclination2, &Inclination3, &Inclination4
};


void BlankBlock(u32 tx, u32 ty) {}

void LeftWall(u32 tx, u32 ty) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)-16+ROOF_MARGIN < ty*32 +32 ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16-ROOF_MARGIN > ty*32 ) {
	if ( ((Player.xpos + Player.xspeed)>>16) < tx*32 + 16 - WALL_MARGIN) {
		Player.xpos = (tx*32 + 16 - WALL_MARGIN)<<16;
		Player.xspeed = 0;
	}
	}
	}
}

void Land(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)-16) < tx*32 +32-WALL_MARGIN ) {
	if ( (((Player.xpos + Player.xspeed)>>16)+16) > tx*32 + WALL_MARGIN ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16) > ty*32 + 16) {
		if (Player.yspeed+Player.yspeed2 > 0) {
			Player.yspeed = 0;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 + 16)<<16;
			Player.on_land = 1;
		}
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.yspeed2 = -GRAVITY*25; Player.on_land=0;}
	}
	}
	}
}

void RightWall(u32 tx, u32 ty) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)-16+ROOF_MARGIN < ty*32 +32 ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16-ROOF_MARGIN > ty*32 ) {
	if ( ((Player.xpos + Player.xspeed)>>16) > tx*32 + 16 + WALL_MARGIN) {
		Player.xpos = (tx*32 + 16 + WALL_MARGIN)<<16;
		Player.xspeed = 0;
	}
	}
	}
}

void Roof(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)-16) < tx*32 +32-WALL_MARGIN ) {
	if ( (((Player.xpos + Player.xspeed)>>16)+16) > tx*32 + WALL_MARGIN ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16) < ty*32+16-ROOF_MARGIN) {
		if (Player.yspeed+Player.yspeed2 < 0) {
			Player.yspeed = 0;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 + 16-ROOF_MARGIN)<<16;
		}
	}
	}
	}
}


void MidLand(u32 tx, u32 ty) {}

void Corner1(u32 tx, u32 ty) {
	LeftWall(tx,ty);
	Land(tx,ty);
}

void Corner2(u32 tx, u32 ty) {
	Roof(tx,ty);
	RightWall(tx,ty);
}

void Corner3(u32 tx, u32 ty) {
	RightWall(tx,ty);
	Land(tx,ty);
}

void Corner4(u32 tx, u32 ty) {
	Roof(tx,ty);
	LeftWall(tx,ty);
}

void Diag1(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) <= tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) >= tx*32  ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16 >= ty*32 + Player.x - tx*32) {
		if (Player.yspeed2 >= 0) {
			Player.yspeed2 = 0;
			Player.yspeed = Player.xspeed;
			Player.ypos = (ty*32 + Player.x - tx*32 - 16)<<16;
			Player.on_land = 2;
		}
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.xspeed = GRAVITY*18; Player.yspeed2 = -GRAVITY*18; Player.on_land=0;}
	}
	}
	}
}

void Diag2(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) <= tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) >= tx*32  ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16 >= ty*32 - Player.x + tx*32 +32-4) {
		if (Player.yspeed2 >= 0) {
			Player.yspeed2 = 0;
			Player.yspeed = -Player.xspeed;
			Player.ypos = (ty*32 - Player.x + tx*32 - 16+32-4)<<16;
			Player.on_land = 3;
		}
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.xspeed = -GRAVITY*18; Player.yspeed2 = -GRAVITY*18; Player.on_land=0;}
	}
	}
	}
}

void Curve1(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) < tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) > tx*32 ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16 > 8 + ty*32 + ((Player.x - tx*32)>>1)) {
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.xspeed = GRAVITY*14; Player.yspeed2 = -GRAVITY*20; Player.on_land=0;}
		if (Player.yspeed2 > 0) {
			Player.yspeed = 0;
			if (Player.xspeed > 0) Player.yspeed = Player.xspeed;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 + ((Player.x - tx*32)>>1) - 16 + 8)<<16;
			Player.on_land = 8;
		}
	}
	}
	}
}

void Curve3(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) < tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) > tx*32 ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16+12 > 8 + ty*32 + ((Player.x - tx*32)>>0)) {
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.xspeed = GRAVITY*14; Player.yspeed2 = -GRAVITY*20; Player.on_land=0;}
		if (Player.yspeed2 > 0) {
			if (Player.xspeed > 0) Player.yspeed = Player.xspeed;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 + ((Player.x - tx*32)>>0) - 16 - 12 + 8)<<16;
			Player.on_land = 10;
		}
	}
	}
	}
}


void Curve2(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) < tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) > tx*32 ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16+12 > 8 + ty*32 + 32+((tx*32 - Player.x)>>0)) {
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.xspeed = -GRAVITY*14; Player.yspeed2 = -GRAVITY*20; Player.on_land=0;}
		if (Player.yspeed2 > 0) {
			if (Player.xspeed < 0) Player.yspeed = -Player.xspeed;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 + ((tx*32 - Player.x)>>0) - 16 - 12 + 8 +32)<<16;
			Player.on_land = 9;
		}
	}
	}
	}
}


void Curve4(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) < tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) > tx*32 ) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16-14 > 8 + ty*32 - ((Player.x - tx*32)>>1)) {
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.xspeed = GRAVITY*14; Player.yspeed2 = -GRAVITY*20; Player.on_land=0;}
		if (Player.yspeed2 > 0) {
			Player.yspeed = 0;
			if (Player.xspeed < 0) Player.yspeed = -Player.xspeed;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 - ((Player.x - tx*32)>>1) - 16 + 8 +14)<<16;
			Player.on_land =11;
		}
	}
	}
	}
} //11

void Inclination1(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) <= tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) >= tx*32) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16 > ty*32 + ((Player.x - tx*32)>>2)) {
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.yspeed2 = -GRAVITY*25; Player.on_land=0;}
		if (Player.yspeed2 > 0) {
			Player.yspeed = 0;
			if (Player.xspeed > 0) Player.yspeed = Player.xspeed;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 + ((Player.x - tx*32)>>2) - 16)<<16;
			Player.on_land = 4;
		}
	}
	}
	}
}

void Inclination3(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) <= tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) >= tx*32) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16-28 > ty*32 + ((Player.x - tx*32)>>2)) {
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.yspeed2 = -GRAVITY*25; Player.on_land=0;}
		if (Player.yspeed2 > 0) {
			Player.yspeed = 0;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 + ((Player.x - tx*32)>>2) - 16 + 28)<<16;
			Player.on_land = 6;
		}
	}
	}
	}
}


void Inclination2(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) <= tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) >= tx*32) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16+6 > ty*32 - ((Player.x - tx*32)>>2)) {
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.yspeed2 = -GRAVITY*25; Player.on_land=0;}
		if (Player.yspeed2 > 0) {
			Player.yspeed = 0;
			if (Player.xspeed < 0) Player.yspeed = -Player.xspeed;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 - ((Player.x - tx*32)>>2) - 16+6)<<16;
			Player.on_land = 5;
		}
	}
	}
	}
}


void Inclination4(u32 tx, u32 ty) {
	if ( (((Player.xpos + Player.xspeed)>>16)) <= tx*32 +32 ) {
	if ( (((Player.xpos + Player.xspeed)>>16)) >= tx*32) {
	if ( ((Player.ypos + Player.yspeed + Player.yspeed2)>>16)+16-32-4 > ty*32 - ((Player.x - tx*32)>>2)) {
		if ( (~KEYPAD & KEY_A) && (Player.on_land!=0))  {Player.yspeed2 = -GRAVITY*25; Player.on_land=0;}
		if (Player.yspeed2 > 0) {
			Player.yspeed = 0;
			Player.yspeed2 = 0;
			Player.ypos = (ty*32 - ((Player.x - tx*32)>>2) - 16 +32+4)<<16;
			Player.on_land = 7;
		}
	}
	}
	}
}





/*  Drawing and display corrections functions  */
extern OAMEntry sprites[128];
extern pRotData rotData;

void DAir() {
	sprites[0].attribute0 = (COLOR_16 | SQUARE | ATTRIB0_DOUBLE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width)));
	sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width)) );
	trans_identity(0);  //9 10 11 12 13
}
void DLand() {
	sprites[0].attribute0 = (COLOR_16 | SQUARE | ATTRIB0_DOUBLE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width)));
	sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width)) );
	if (Player.orientation > 0) trans_identity(0);
	else trans_identity_f(0);
}
void DDiag1() {
	sprites[0].attribute0 = (COLOR_16 | SQUARE | ATTRIB0_DOUBLE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width - 4)));
	sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width + 3)) );
	if (Player.orientation > 0) trans_rotate_45(0);
	else trans_rotate_45_f(0);
}
void DDiag2() {
	sprites[0].attribute0 = (COLOR_16 | SQUARE | ATTRIB0_DOUBLE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width - 2)));
	sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width - 2)) );
	if (Player.orientation > 0) trans_rotate_m45(0);
	else trans_rotate_m45_f(0);
}
void DInc1() {
	sprites[0].attribute0 = (COLOR_16 | SQUARE | ATTRIB0_DOUBLE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width-2)));
	sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width)) );
	if (Player.orientation > 0) trans_rotate_15(0);
	else trans_rotate_15_f(0);
}
void DInc2() {
	sprites[0].attribute0 = (COLOR_16 | SQUARE | ATTRIB0_DOUBLE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width-2)));
	sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width)) );
	if (Player.orientation > 0) trans_rotate_m15(0);
	else trans_rotate_m15_f(0);
}
void DCurve1() {
	sprites[0].attribute0 = (COLOR_16 | SQUARE | ATTRIB0_DOUBLE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width-2)));
	sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width)) );
	if (Player.orientation > 0) trans_rotate_25(0);
	else trans_rotate_25_f(0);
}
void DCurve2() {
	sprites[0].attribute0 = (COLOR_16 | SQUARE | ATTRIB0_DOUBLE | ROTATION_FLAG | (0x00ff&(Player.y - vp.ymin - Player.width-2)));
	sprites[0].attribute1 = (SIZE_32 | ROTDATA(0) | (0x01ff&(Player.x -vp.xmin - Player.width)) );
	if (Player.orientation > 0) trans_rotate_m25(0);
	else trans_rotate_m25_f(0);
}

void (*CharDisplay[12])()  = {
	//0       1       2         3
	&DAir, &DLand, &DDiag1, &DDiag2,
	//4      5        6         7         //Inclinations1-4
	&DInc1,  &DInc2, &DInc1,  &DInc2,
	//8      9        10        11        //Curves 1-4
	&DCurve1, &DCurve2, &DCurve1, &DCurve2
};

