
#include "gba_types.h"
#include "gba_mem.h"
#include "gba_const.h"
#include "level_loader.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef CHARACTER_H
#define CHARACTER_H

typedef struct Character {
	s32 x, y;
	u32 frame;
	s32 xspeed, yspeed;
	s32 yspeed2;  //speed caused by gravity is counted here
	s32 xaccel, yaccel;
	s32 xpos, ypos;
	u32 width, height;
	s32 orientation;
	u32 on_land;
	u32 ball;
	s32 up_down;
}Character;


#define GRAVITY 11000
#define MAX_GRAVITY_SPEED 500000

#define SPEED_FACTOR 7000 
#define SPEED_FACTOR_AIR 3000
#define MAX_USER_SPEED 2<<16

#define K_COEF 1<<10
#define K_COEF_START 2<<7

#define NEARLY_QUIET 1<<15

void (*CharDisplay[12])();

/*   This are the physics funcs   */
void PlayerPhysics(tlevel * level);
void PlayerCollision(tlevel * level);

void BlankBlock(u32 tx, u32 ty);
void LeftWall(u32 tx, u32 ty);
void Land(u32 tx, u32 ty);
void RightWall(u32 tx, u32 ty);
void Roof(u32 tx, u32 ty);

void MidLand(u32 tx, u32 ty);
void Corner1(u32 tx, u32 ty);
void Corner2(u32 tx, u32 ty);
void Corner3(u32 tx, u32 ty);
void Corner4(u32 tx, u32 ty);

void Diag1(u32 tx, u32 ty);
void Diag2(u32 tx, u32 ty);

void Curve1(u32 tx, u32 ty);
void Curve2(u32 tx, u32 ty);
void Curve3(u32 tx, u32 ty);
void Curve4(u32 tx, u32 ty);

void Inclination1(u32 tx, u32 ty);
void Inclination2(u32 tx, u32 ty);
void Inclination3(u32 tx, u32 ty);
void Inclination4(u32 tx, u32 ty);

void (*ProcessBlock[20])(u32 tx, u32 ty);


#endif


