
#include "gba_types.h"
#include "gba_mem.h"
#include "gba_const.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "level_loader.h"
#include "character.h"

u32 lastcol0 = -1, lastrow0 = -1;
u32 lastcol1 = -1, lastrow1 = -1;

extern VIEWPORT vp;
extern Character Player;

inline void bgs_copy_row (u16 * datap, u16 * sb,u32 mapwidth, u32 x, u32 y) {
	u32 ix, x0= x&31;

	u16 *srcL= (u16*)&datap[y*mapwidth + x];
	u16 *dstL= (u16*)&sb[(y&31)*32 + x0];

	for(ix=x0; ix<32; ix++)
		*dstL++= *srcL++;

	dstL -= 32;

	for(ix=0; ix<x0; ix++)
		*dstL++= *srcL++;
}

inline void bgs_copy_col (u16 * datap, u16 * sb,u32 mapwidth, u32 x, u32 y) {
	u32 iy, y0= y&31;

	u16 *srcL= (u16*)&datap[y*mapwidth + x];
	u16 *dstL= (u16*)&sb[y0*32 + (x&31)];

	for(iy=y0; iy<32; iy++) {*dstL= *srcL;dstL += 32;srcL += mapwidth;}

	dstL -= 1024;

	for(iy=0; iy<y0; iy++){*dstL= *srcL;dstL += 32;srcL += mapwidth;}
}

inline void ViewportAdjust (u32 mapwidth, u32 mapheight) {
	vp.xmax = vp.x + 120;
	vp.xmin = vp.x - 120;
	vp.ymax = vp.y + 80;
	vp.ymin = vp.y - 80;

	if (vp.xmin < 0) {vp.xmin = 0; vp.xmax = 240; vp.x = 120;}
	if (vp.ymin < 0) {vp.ymin = 0; vp.ymax = 160; vp.y = 80;}

	s32 mapw = mapwidth << 3, maph = mapheight << 3;
	if (vp.xmax > mapw) {vp.xmin = mapw - 240; vp.xmax = mapw; vp.x = mapw - 120;}
	if (vp.ymax > maph) {vp.ymin = maph - 160; vp.ymax = maph; vp.y = maph - 80;}
}



void BGScroll(u16 * datap, u16 * sb, u32 mapwidth, u32 mapheight) {
	ViewportAdjust (mapwidth, mapheight);
	
	u32 mapy0,mapx0,mapy1,mapx1;

	mapy0 = (vp.ymin >> 3);
	mapx0 = (vp.xmin >> 3);
	mapy1 = (vp.ymax >> 3);
	mapx1 = (vp.xmax >> 3);

	if (mapy0 != lastrow0) bgs_copy_row (datap, sb, mapwidth, mapx0, mapy0);
	if (mapy1 != lastrow1) bgs_copy_row (datap, sb, mapwidth, mapx0, mapy1);
	lastrow0 = mapy0; lastrow1 = mapy1;

	if (mapx0 != lastcol0) bgs_copy_col (datap, sb, mapwidth, mapx0, mapy0);
	if (mapx1 != lastcol1) bgs_copy_col (datap, sb, mapwidth, mapx1, mapy0);
	lastcol0 = mapx0; lastcol1 = mapx1;

	BACKGROUND2_X_SCROLL = vp.xmin & 0xFF; //modulo
	BACKGROUND2_Y_SCROLL = vp.ymin & 0xFF;
}

inline void ViewportCenter (u32 x, u32 y) {
	vp.x = x;
	vp.y = y;
}

////////// TILESET
// PALETE 256 colors * 2 byte           : 512 bytes
// u32 NUM_TILES                        : 4 bytes
// FILLER                               : 508 bytes
//                                      -- 1024 bytes --
// TILE (1bytes/pixel) 8x8 = 64 pixels = 64 bytes


////////// LEVEL / BG
// u32 width, height                    : 8 bytes
// each tile index 2 bytes

void LoadTileset (u8 * tileset, u8 * charblock) {
	//Copy palette data
	memcpy( (u8 *)(0x5000000), &tileset[0], 512);

	u32 numt = *(u32*)&tileset[512];

	memcpy ( charblock, &tileset[1024], numt * 64);
}


void LoadLevel(u8 * level_data, u8 * screenblock, u8 * collision, tlevel * level) {
	level->level_buffer_screenblock = (u16*)&level_data[8];
	level->screenblock = (u16*)screenblock;
	level->collision = collision;
	
	//Extract Width, Height, NumTiles
	level->width = (u32)(*(u16*)&level_data[0]);
	level->height = (u32)(*(u16*)&level_data[4]);

	level->width32 = level->width >> 2;
	level->height32 = level->height >> 2;
}

void SetCharacterPos(u32 x, u32 y, tlevel * level) {
	//Set the player Position
	Player.yaccel = Player.xaccel = 0;
	Player.yspeed = Player.xspeed = 0;
	Player.xpos = x << 16;
	Player.ypos = y << 16;
	Player.x = x;
	Player.y = y;
	Player.width = Player.height = 32;
	Player.orientation = 1;
	
	//Center The Viewport
	ViewportCenter (x,y);
	ViewportAdjust(level->width,level->height);

	//Copy the screenblocks
	u32 basex = vp.xmin >> 3;
	u32 basey = vp.ymin >> 3;   //Calculate the top block

	u32 i;   //we just copy 32 rows
	for (i = 0; i < 32; i++) bgs_copy_row((u16*)&level->level_buffer_screenblock[0],(u16*)level->screenblock,level->width,basex,basey+i);
}


void Copy64x64 (tlevel * level) {
	u32 i, j, c = 0;
	for (i = 0; i < 32; i++) {
		for (j = 0; j < 32; j++) {
			level->screenblock[c++] = level->level_buffer_screenblock[j+i*64];
		}
	}
	for (i = 0; i < 32; i++) {
		for (j = 32; j < 64; j++) {
			level->screenblock[c++] = level->level_buffer_screenblock[j+i*64];
		}
	}
	for (i = 32; i < 64; i++) {
		for (j = 0; j < 32; j++) {
			level->screenblock[c++] = level->level_buffer_screenblock[j+i*64];
		}
	}
	for (i = 32; i < 64; i++) {
		for (j = 32; j < 64; j++) {
			level->screenblock[c++] = level->level_buffer_screenblock[j+i*64];
		}
	}
}



