
#include "gba_types.h"
#include "gba_mem.h"
#include "gba_const.h"
#include "aux.h"
#include <stdlib.h>
#include <stdio.h>

#include "data_read.h"
#include "character.h"
#include "level_loader.h"

#include "init.h"
#include "trans.h"

extern void VBlankIntrWait();

OAMEntry sprites[128];
pRotData rotData = (pRotData)sprites;

Character Player;
VIEWPORT vp;

RotInfo rot_info;

//ScreenBlock  24,25,26,27 Background
//             31 Level

int AgbMain() {
	int i;
	u16 * pal = (u16*)BACKGROUND_PALETTE;

	for (i = 0; i < 100; i++) ColorScreen(0x0000);

	PrintLogo();

	DISPLAY_CONTROL = SCREEN_OFF;

	tlevel level1, back;
	LoadTileset ((u8*)GetFilePointer("level3.tileset"),(u8 *)BACKGROUND_CHARACTER(0));

	LoadLevel ((u8*)GetFilePointer("level3.level"),(u8 *)BACKGROUND_MAP(31),(u8*)GetFilePointer("level3.collision"),&level1);
	LoadLevel ((u8*)GetFilePointer("background.level"),(u8 *)BACKGROUND_MAP(24),NULL,&back);
	SetCharacterPos(1200,100,&level1);

	Copy64x64(&back);

	
	u8 * palette_sonic = (u8*)GetFilePointer("sonicsprite.sprite");
	u16 * bitmap_sonic = (u16*)&palette_sonic[32];
	memcpy( (u8 *)0x5000200, palette_sonic, 32);
	memcpy ( (u16*)(0x6010000), &bitmap_sonic[0], 896*32/2);
	
	u16 * pale = (u16*)(0x5000000);
	u32 r, g , b;
	for (i = 0; i < 512; i++) {
		r = (pale[i]&31) + 10;
		g = ((pale[i]>>5)&31) + 10;
		b = ((pale[i]>>10)&31) + 10;
		if (r > 31) r = 31;
		if (g > 31) g = 31;
		if (b > 31) b = 31;
		pale[i] = r | (g<<5) | (b<<10);
	}
	
	pal[0]=color16(0,0,0);   //undrawn color
	hide_all_sprites();
	trans_identity(0);


	DISPLAY_CONTROL = OBJECTS | MODE0 | BACKGROUND2 | BACKGROUND3 | SPRITE_1D;
	REG_VCOUNT |= (1 << 3);   //enable Vblank flag
	BACKGROUND2_CONTROL = BACKGROUND_WHICH_CHARACTER(0) | BACKGROUND_WHICH_MAP(31) | BACKGROUND_COLOR_256;
	BACKGROUND2_X_SCROLL = 0;
	BACKGROUND2_Y_SCROLL = 0;

	BACKGROUND3_CONTROL = BACKGROUND_WHICH_CHARACTER(0) | BACKGROUND_WHICH_MAP(24) | BACKGROUND_COLOR_256 | TEXT_SIZE_512x512;
	BACKGROUND3_X_SCROLL = 0;
	BACKGROUND3_Y_SCROLL = 0;



	i = 0;
	u32 anim_frame = 0;
	u32 last_state = 0;
	while (1) {
		BACKGROUND3_X_SCROLL = vp.xmin>>3;	//Background scrolling
		BACKGROUND3_Y_SCROLL = vp.ymin>>3;

		PlayerPhysics(&level1); 		//Physics

		vsync();				//Wait to write VRAM

		BGScroll((u16*)level1.level_buffer_screenblock,(u16 *)(BACKGROUND_MAP(31)),level1.width,level1.height);	//Update level

		CharDisplay[Player.on_land]();			//Write main sprite attributes

		//Process animation frame
		if (last_state != Player.on_land) anim_frame = 0;

		if (Player.on_land == 0) {		//Jumping
			anim_frame += 40000;
			if (((anim_frame>>16) & 0x1)!=0) sprites[0].attribute2 = (((anim_frame/2)>>16) + 10)<<4;
			else sprites[0].attribute2 = 9<<4;
			if (anim_frame >= (8<<16)) anim_frame = 0;
		}else{
			if (Player.xspeed == 0) sprites[0].attribute2 = 0;	//Standing
			else {

			sprites[0].attribute2 = ((i+5) / 5 )<<4;
	//		sprites[0].attribute2 = 0;
	
			i++;
			if ((i / 5 ) >= 8) i = 0;
			}
		}
	
		u32 loop;
		u16* temp;
		temp = (u16*)sprites;
		for(loop = 0;loop < 4*4; loop++) {  //copy sprite info
			OAMmem[loop] = temp[loop];
		}
		last_state =  Player.on_land;
	}

	return 0;
}


