
#include <iostream>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <vector>

using namespace std;

typedef struct tile {
	unsigned char data [8*8];
} tile;

bool tile_compare(const unsigned char * a, const unsigned char * b) {
	for (int i = 0; i < 64; i++) if (a[i] != b[i]) return false;
	return true;
}

void fliph(const unsigned char * a, unsigned char * b) {
	for (int i = 0; i < 8; i++) {
		for (int j = 0; j < 8; j++) {
			b[8*j+8-i-1] = a[8*j+i];
		}
	}
}

void flipv(const unsigned char * a, unsigned char * b) {
	for (int i = 0; i < 8; i++) {
		for (int j = 0; j < 8; j++) {
			b[j+(8-i-1)*8] = a[j+i*8];
		}
	}
}


//GBA uses BGR!!!!!!!!!
int f_24to16 (int color) {
	int red = color >> 16;
	int green = (color >> 8) & 0xFF;
	int blue = color & 0xFF;
	return ((red >> 3)<< 0) | ((green >> 3) << 5) | ((blue >> 3) <<10);
}

////////// TILESET
// PALETE 256 colors * 2 byte           : 512 bytes
// u32 NUM_TILES                        : 4 bytes
// FILLER                               : 508 bytes

//                                      -- 1024 bytes --

// TILE (1bytes/pixel) 8x8 = 64 pixels = 64 bytes

int main (int argc, const char *argv[]) {
	//tile_tool   input.bmp   output.tileset    transparent_color
	string input(argv[1]), output(argv[2]);
	int trans_color = atoi(argv[3]); trans_color = f_24to16(trans_color);

	/// READ THE INPUT BMP FILE
	FILE * pFile = fopen (&input[0],"rb");
	if (pFile == NULL) { cout << "Cannot open input file: " << input << endl; return 0; }
	fseek(pFile, 0, SEEK_END);
	int fis = ftell(pFile);
	fseek(pFile, 0, SEEK_SET);
	char * inputbuffer = new char [fis];
	fread(&inputbuffer[0],1,fis,pFile);
	fclose (pFile);

	short bpp;
	int width, height;
	width = *(int *)&inputbuffer[18];
	height = *(int *)&inputbuffer[22];
	bpp = *(short *)&inputbuffer[28];

	cout << "The bitmap is " << width << "x" << height << " and " << bpp << "bpp" << endl;
	if (width % 8 != 0 or height % 8 != 0) {cout << "The image dimensions are not multiple of 8!" << endl; return 0;}

	int counter =0;
	unsigned char * ibuffer = (unsigned char *)&inputbuffer[*(int*)&inputbuffer[10]];  //Pointer to the input data
	unsigned short * obuffer =  new unsigned short [width*height];  //Pointer to a new buffer 16 bit color
	unsigned short * obuffer2 =  new unsigned short [width*height];  //Pointer to a new buffer 16 bit color (flip)
	unsigned char * obuffer3 = new unsigned char [width*height];  //The buffer with the indexed color image

	for (int i = 0; i <  width*height*bpp/8; i=i+(bpp/8)) {
		obuffer[counter++] = (ibuffer[i+2] >> 3) | ((ibuffer[i+1] >> 3) << 5) | ((ibuffer[i+0] >> 3) << 10); }

	//We invert the lines, as the BMP is fliped vertically
	int k = 0;
	for (int i = height-1; i >= 0; i--) {
		memcpy(&obuffer2[k*width],&obuffer[i*width],width*2);  //the file
		k++;
	}
	delete [] obuffer;

	unsigned short palette[256];
	palette[0] = trans_color; //the original color, needed when indexing

	//Now we have to index the color
	int num_colors = 1;   //the first color is transparent!
	for (int i = 0; i < counter; i++) {
		//look up the color, is it already in the list?
		bool listed = false;
		int index = 0;
		if (obuffer2[i] == trans_color) {
			obuffer3[i] = 0;
		}else{
			for (int j = 1; j < num_colors; j++) {
				if (obuffer2[i] == palette[j]) {
					listed = true;
					index = j;
					break;
				}
			}
			if (not listed) {
				palette[num_colors] = obuffer2[i];
				index = num_colors;
				num_colors++;
			}
			obuffer3[i] = index;
		}	
	}
	delete [] obuffer2;

	cout << num_colors << " colors found!" << endl;

	//CREATE A LIST WITH ALL THE 8x8 TILES

	int num_tiles = width*height/64;
	tile atile;
	vector <tile> tilelist(num_tiles);
	counter = 0;
	int twidth = width/8;
	int theight = height/8;
	
	for (int j = 0; j < theight; j++) {
		for (int i = 0; i < twidth; i++) {
			for (int k = 0; k < 8; k++) {
				memcpy (&atile.data[8*k],&obuffer3[i*8+j*width*8+k*width],8);
			}
			memcpy(&tilelist[counter].data[0],&atile.data[0],8*8);
			counter ++;
		}
	}
	delete [] obuffer3;
	
	//NOW REMOVE THE REPEATED TILES
	vector <tile> tilelist2(num_tiles);
	tile temp;
	int num_tiles2 = 0;
	for (int i = 0; i < num_tiles; i++) {
		bool found = false;
		for (int j = 0; j < num_tiles2; j++) {
			if (tile_compare(&tilelist[i].data[0],&tilelist2[j].data[0])) {  //the same tile
				found = true;
				break;
			}
			fliph(&tilelist[i].data[0],&temp.data[0]);
			if (tile_compare(&temp.data[0],&tilelist2[j].data[0])) {   //fliped H
				found = true;
				break;
			}
			flipv(&tilelist[i].data[0],&temp.data[0]);
			if (tile_compare(&temp.data[0],&tilelist2[j].data[0])) {   //fliped V
				found = true;
				break;
			}
			flipv(&tilelist[i].data[0],&temp.data[0]);
			fliph(&temp.data[0],&temp.data[0]);
			if (tile_compare(&temp.data[0],&tilelist2[j].data[0])) {   //fliped V & H
				found = true;
				break;
			}
		}
		if (not found) {	//add to list
			memcpy (&tilelist2[num_tiles2].data[0],&tilelist[i].data[0],64);
			num_tiles2++;
		}
	}
	cout << "Found " << num_tiles2 << " unique tiles" << endl;
	
	//NOW WE ARE READY TO WRITE ALL THE OUTPUT FILE!
	FILE * pFile2 = fopen (&output[0],"w");
	fwrite (&palette[0],1,512,pFile2);
	unsigned int aux = num_tiles2;
	fwrite (&aux,1,4,pFile2);
	unsigned char zero[508]; memset(&zero[0],0,508);
	fwrite (&zero[0],1,508,pFile2);
	for (int i = 0; i < num_tiles2; i++) {
		fwrite(&tilelist2[i].data[0],1,64,pFile2);
	}
	fclose(pFile2);

	cout << "DONE" << endl;
	delete [] inputbuffer;
	return 0;
}


